package cn.gygxzc.envir.encrypt;

import cn.gygxzc.cloud.tina.auth.encrypt.AESUtils;
import cn.gygxzc.envir.core.CoreConstKt;
import cn.gygxzc.envir.core.exceptions.InvalidRequestException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.core.MethodParameter;
import org.springframework.http.MediaType;
import org.springframework.http.server.ServerHttpRequest;
import org.springframework.http.server.ServerHttpResponse;
import org.springframework.web.bind.annotation.ControllerAdvice;
import org.springframework.web.servlet.mvc.method.annotation.ResponseBodyAdvice;

import java.lang.reflect.Method;

/**
 * @author tuonina
 * @createTime 2019/2/26
 */
@ControllerAdvice("cn.gygxzc.envir")
public class EncodeResponseBodyAdvice implements ResponseBodyAdvice {
    private Logger logger = LoggerFactory.getLogger(EncodeResponseBodyAdvice.class);

    @Autowired
    private ObjectMapper objectMapper;

    @Override
    public boolean supports(MethodParameter returnType, Class converterType) {
        Method method = returnType.getMethod();
        if (method == null) return false;
        SerializedField serializedField = method.getAnnotation(SerializedField.class);
        return serializedField != null && serializedField.encode();
    }

    @Override
    public Object beforeBodyWrite(Object body, MethodParameter returnType,
                                  MediaType selectedContentType, Class selectedConverterType,
                                  ServerHttpRequest request, ServerHttpResponse response) {
        String clientKey = request.getHeaders().getFirst(CoreConstKt.REQ_HEADER_CLIENT);
        if (StringUtils.isBlank(clientKey)) {
            logger.info("beforeBodyWrite 缺少请求头：{}", CoreConstKt.REQ_HEADER_CLIENT);
            throw new InvalidRequestException("缺少请求头!");
        }
        try {
            String result = objectMapper.writeValueAsString(body);
            return AESUtils.encode(result, clientKey);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }
}
